import NDK, { getRelayListForUser, NDKList, NDKUser } from '@nostr-dev-kit/ndk'
import { kinds } from 'nostr-tools'
import { MuteLists } from '../types'
import { UserProfile } from '../types/user'
import { hexToNpub, log, LogType, npubToHex } from '../utils'

/**
 * Singleton class to manage metadata operations using NDK.
 */
export class MetadataController {
  private static instance: MetadataController
  private ndk: NDK
  private usersMetadata = new Map<string, UserProfile>()
  public adminNpubs: string[]
  public adminRelays = new Set<string>()

  private constructor() {
    this.ndk = new NDK({
      explicitRelayUrls: [
        'wss://user.kindpag.es',
        'wss://purplepag.es',
        'wss://relay.damus.io/',
        import.meta.env.VITE_APP_RELAY
      ]
    })
    this.ndk.connect()

    this.adminNpubs = import.meta.env.VITE_ADMIN_NPUBS.split(',')
  }

  private setAdminRelays = async () => {
    const promises = this.adminNpubs.map((npub) => {
      const hexKey = npubToHex(npub)
      if (!hexKey) return null

      return getRelayListForUser(hexKey, this.ndk)
        .then((ndkRelayList) => {
          if (ndkRelayList) {
            ndkRelayList.writeRelayUrls.forEach((url) =>
              this.adminRelays.add(url)
            )
          }
        })
        .catch((err) => {
          log(
            true,
            LogType.Error,
            `❌ Error occurred in getting the instance of NDKRelayList for npub: ${npub}`,
            err
          )
        })
    })

    await Promise.allSettled(promises)
  }

  /**
   * Provides the singleton instance of MetadataController.
   *
   * @returns The singleton instance of MetadataController.
   */
  public static async getInstance(): Promise<MetadataController> {
    if (!MetadataController.instance) {
      MetadataController.instance = new MetadataController()

      MetadataController.instance.setAdminRelays()
    }
    return MetadataController.instance
  }

  /**
   * Finds metadata for a given pubkey.
   *
   * @param hexKey - The pubkey to search for metadata.
   * @returns A promise that resolves to the metadata event.
   */
  public findMetadata = async (pubkey: string): Promise<UserProfile> => {
    const npub = hexToNpub(pubkey)

    const cachedMetadata = this.usersMetadata.get(npub)
    if (cachedMetadata) {
      return cachedMetadata
    }

    const user = new NDKUser({ npub })
    user.ndk = this.ndk

    const userProfile = await user.fetchProfile()
    if (userProfile) {
      this.usersMetadata.set(npub, userProfile)
    }

    return userProfile
  }

  /**
   * Finds metadata for admin user.
   *
   * @returns A promise that resolves to the metadata event.
   */
  public findAdminMetadata = async (): Promise<UserProfile> => {
    return this.findMetadata(this.adminNpubs[0])
  }

  public findWriteRelays = async (hexKey: string) => {
    const ndkRelayList = await getRelayListForUser(hexKey, this.ndk)

    if (!ndkRelayList) {
      throw new Error(`Couldn't found user's relay list`)
    }

    return ndkRelayList.writeRelayUrls
  }

  public getAdminsMuteLists = async (): Promise<MuteLists> => {
    // Create a Set to collect all unique muted authors

    const mutedAuthors = new Set<string>()

    // Create an array of promises to fetch mute lists for each npub
    const promises = this.adminNpubs.map(async (npub) => {
      const hexKey = npubToHex(npub)
      if (!hexKey) return

      const muteListEvent = await this.ndk.fetchEvent({
        kinds: [kinds.Mutelist],
        authors: [hexKey]
      })

      if (muteListEvent) {
        const list = NDKList.from(muteListEvent)

        list.items.forEach((item) => {
          // Add muted authors to the Set directly
          if (item[0] === 'p') {
            mutedAuthors.add(item[1])
          }
        })
      }
    })

    await Promise.allSettled(promises)

    return {
      authors: Array.from(mutedAuthors),
      eventIds: []
    }
  }
}
