import { Event, Filter, Relay } from 'nostr-tools'
import { log, LogType, normalizeWebSocketURL, timeout } from '../utils'
import { MetadataController } from './metadata'

/**
 * Singleton class to manage relay operations.
 */
export class RelayController {
  private static instance: RelayController
  private debug = true
  public connectedRelays: Relay[] = []

  private constructor() {}

  /**
   * Provides the singleton instance of RelayController.
   *
   * @returns The singleton instance of RelayController.
   */
  public static getInstance(): RelayController {
    if (!RelayController.instance) {
      RelayController.instance = new RelayController()
    }
    return RelayController.instance
  }

  public connectRelay = async (relayUrl: string) => {
    const relay = this.connectedRelays.find(
      (relay) =>
        normalizeWebSocketURL(relay.url) === normalizeWebSocketURL(relayUrl)
    )
    if (relay) {
      // already connected, skip
      return relay
    }

    return await Relay.connect(relayUrl)
      .then((relay) => {
        log(this.debug, LogType.Info, `✅ nostr (${relayUrl}): Connected!`)
        this.connectedRelays.push(relay)
        return relay
      })
      .catch((err) => {
        log(
          this.debug,
          LogType.Error,
          `❌ nostr (${relayUrl}): Connection error!`,
          err
        )
        return null
      })
  }

  /**
   * Publishes an event to multiple relays.
   *
   * This method connects to the application relay and a set of write relays
   * obtained from the `MetadataController`. It then publishes the event to
   * all connected relays and returns a list of relays where the event was successfully published.
   *
   * @param event - The event to be published.
   * @returns A promise that resolves to an array of URLs of relays where the event was published,
   *          or an empty array if no relays were connected or the event could not be published.
   */
  publish = async (event: Event): Promise<string[]> => {
    // Connect to the application relay specified by environment variable
    const appRelayPromise = this.connectRelay(import.meta.env.VITE_APP_RELAY)

    // todo: window.nostr.getRelays() is not implemented yet in nostr-login, implement the logic once its done

    const metadataController = await MetadataController.getInstance()

    // Retrieve the list of write relays for the event's public key
    // Use a timeout to handle cases where retrieving write relays takes too long
    const writeRelaysPromise = metadataController.findWriteRelays(event.pubkey)

    log(this.debug, LogType.Info, `ℹ Finding user's write relays`)

    // Use Promise.race to either get the write relay URLs or timeout
    const writeRelayUrls = await Promise.race([
      writeRelaysPromise,
      timeout() // This is a custom timeout function that rejects the promise after a specified time
    ]).catch((err) => {
      log(this.debug, LogType.Error, err)
      return [] as string[] // Return an empty array if an error occurs
    })

    // push admin relay urls obtained from metadata controller to writeRelayUrls list
    metadataController.adminRelays.forEach((url) => {
      writeRelayUrls.push(url)
    })

    // Connect to all write relays obtained from MetadataController
    const relayPromises = writeRelayUrls.map((relayUrl) =>
      this.connectRelay(relayUrl)
    )

    // Wait for all relay connections to settle (either fulfilled or rejected)
    await Promise.allSettled([appRelayPromise, ...relayPromises])

    // Check if any relays are connected; if not, log an error and return null
    if (this.connectedRelays.length === 0) {
      log(this.debug, LogType.Error, 'No relay is connected!')
      return []
    }

    const publishedOnRelays: string[] = [] // List to track which relays successfully published the event

    // Create a promise for publishing the event to each connected relay
    const publishPromises = this.connectedRelays.map((relay) => {
      log(
        this.debug,
        LogType.Info,
        `⬆️ nostr (${relay.url}): Sending event:`,
        event
      )

      return Promise.race([
        relay.publish(event), // Publish the event to the relay
        timeout(30000) // Set a timeout to handle cases where publishing takes too long
      ])
        .then((res) => {
          log(
            this.debug,
            LogType.Info,
            `⬆️ nostr (${relay.url}): Publish result:`,
            res
          )
          publishedOnRelays.push(relay.url) // Add the relay URL to the list of successfully published relays
        })
        .catch((err) => {
          log(
            this.debug,
            LogType.Error,
            `❌ nostr (${relay.url}): Publish error!`,
            err
          )
        })
    })

    // Wait for all publish operations to complete (either fulfilled or rejected)
    await Promise.allSettled(publishPromises)

    // Return the list of relay URLs where the event was published
    return publishedOnRelays
  }

  /**
   * Asynchronously retrieves multiple event from a set of relays based on a provided filter.
   * If no relays are specified, it defaults to using connected relays.
   *
   * @param {Filter} filter - The filter criteria to find the event.
   * @param {string[]} [relays] - An optional array of relay URLs to search for the event.
   * @returns {Promise<Event | null>} - Returns a promise that resolves to the found event or null if not found.
   */
  fetchEvents = async (
    filter: Filter,
    relays: string[] = []
  ): Promise<Event[]> => {
    // add app relay to relays array
    relays.push(import.meta.env.VITE_APP_RELAY)

    const metadataController = await MetadataController.getInstance()
    // add admin relays to relays array
    metadataController.adminRelays.forEach((url) => {
      relays.push(url)
    })

    // Connect to all specified relays
    const relayPromises = relays.map((relayUrl) => this.connectRelay(relayUrl))
    await Promise.allSettled(relayPromises)

    // Check if any relays are connected
    if (this.connectedRelays.length === 0) {
      log(this.debug, LogType.Error, 'No relay is connected to fetch events!')
      throw new Error('No relay is connected to fetch events!')
    }

    const events: Event[] = []
    const eventIds = new Set<string>() // To keep track of event IDs and avoid duplicates

    // Create a promise for each relay subscription
    const subPromises = this.connectedRelays.map((relay) => {
      return new Promise<void>((resolve) => {
        // Subscribe to the relay with the specified filter
        const sub = relay.subscribe([filter], {
          // Handle incoming events
          onevent: (e) => {
            // Add the event to the array if it's not a duplicate
            if (!eventIds.has(e.id)) {
              eventIds.add(e.id) // Record the event ID
              events.push(e) // Add the event to the array
            }
          },
          // Handle the End-Of-Stream (EOSE) message
          oneose: () => {
            sub.close() // Close the subscription
            resolve() // Resolve the promise when EOSE is received
          }
        })
      })
    })

    // Wait for all subscriptions to complete
    await Promise.allSettled(subPromises)

    // It is possible that different relays will send different events and events array may contain more events then specified limit in filter
    // To fix this issue we'll first sort these events and then return only limited events
    if (filter.limit) {
      // Sort events by creation date in descending order
      events.sort((a, b) => b.created_at - a.created_at)

      return events.slice(0, filter.limit)
    }

    return events
  }

  /**
   * Asynchronously retrieves an event from a set of relays based on a provided filter.
   * If no relays are specified, it defaults to using connected relays.
   *
   * @param {Filter} filter - The filter criteria to find the event.
   * @param {string[]} [relays] - An optional array of relay URLs to search for the event.
   * @returns {Promise<Event | null>} - Returns a promise that resolves to the found event or null if not found.
   */
  fetchEvent = async (
    filter: Filter,
    relays: string[] = []
  ): Promise<Event | null> => {
    const events = await this.fetchEvents(filter, relays)

    // Sort events by creation date in descending order
    events.sort((a, b) => b.created_at - a.created_at)

    // Return the most recent event, or null if no events were received
    return events[0] || null
  }
}
